@@ -34,7 +34,7 @@ Fsdb - a flat-text database for shell scripting
 
 
 =cut
-our $VERSION = '2.56';
+our $VERSION = '2.58';
 
 =head1 SYNOPSIS
 
@@ -234,19 +234,18 @@ L<http://www.isi.edu/~johnh/SOFTWARE/FSDB/index.html>.
 
 =head1 WHAT'S NEW
 
-=head2 2.56, 2015-02-03
-fix against Getopt::Long-2.43's stricter error checkign
+=head2 2.58, 2015-04-30
+Bugfix in L<dbmerge>
 
 =over 4
 
 =item BUG FIX
 
-Internal argument parsing uses Getopt::Long, but mixed pass-through and E<lt>E<gt>.
-Bug reported by Petr Pisar at L<https://bugzilla.redhat.com/show_bug.cgi?id=1188538>.
-
-=item BUG FIX
-
-Added missing BuildRequires for C<XML::Simple>.
+Fixed a case where L<dbmerge> suffered mojobake in endgame mode.
+This bug surfaced when L<dbsort> was applied to large files
+(big enough to require merging) with unicode in them;
+the symptom was soemthing like:
+  Wide character in print at /usr/lib64/perl5/IO/Handle.pm line 420, <GEN12> line 111.
 
 =back
 
@@ -3255,6 +3254,21 @@ These are now correctly spelled.
 
 =back
 
+=head2 2.56, 2015-02-03
+fix against Getopt::Long-2.43's stricter error checkign
+
+=over 4
+
+=item BUG FIX
+
+Internal argument parsing uses Getopt::Long, but mixed pass-through and E<lt>E<gt>.
+Bug reported by Petr Pisar at L<https://bugzilla.redhat.com/show_bug.cgi?id=1188538>.a
+
+=item BUG FIX
+
+Added missing BuildRequires for C<XML::Simple>.
+
+=back
 
 
 =head1 AUTHOR
