@@ -61,7 +61,7 @@ use 5.010;
 use Carp qw(carp croak);
 use IO::Handle;
 use IO::File;
-use Getopt::Long qw(GetOptionsFromArray :config no_ignore_case bundling permute passthrough);
+use Getopt::Long qw(GetOptionsFromArray :config no_ignore_case bundling permute);
 
 use Fsdb::IO::Reader;
 use Fsdb::IO::Writer;
