@@ -113,6 +113,11 @@ Also uses environment variable TMPDIR, if -T is
 not specified.
 Default is /tmp.
 
+=item B<-k KeyField>
+
+Do multi-stats, grouped by each key.
+Assumes keys are sorted.  (Use dbmultistats to guarantee sorting order.)
+
 
 =back
 
@@ -247,7 +252,7 @@ Create a new dbcolstats object, taking command-line arguments.
 
 =cut
 
-sub new ($@) {
+sub new($@) {
     my $class = shift @_;
     my $self = $class->SUPER::new(@_);
     bless $self, $class;
@@ -266,7 +271,7 @@ Internal: set up defaults.
 
 =cut
 
-sub set_defaults ($) {
+sub set_defaults($) {
     my($self) = @_;
     $self->SUPER::set_defaults();
     $self->{_target_column} = undef;
@@ -279,6 +284,7 @@ sub set_defaults ($) {
     $self->{_include_non_numeric} = undef;
     $self->{_fscode} = undef;
     $self->{_max_parallelism} = undef;
+    $self->{_key_column} = undef;
     $self->set_default_tmpdir;
 }
 
@@ -290,7 +296,7 @@ Internal: parse command-line arguments.
 
 =cut
 
-sub parse_options ($@) {
+sub parse_options($@) {
     my $self = shift @_;
 
     my(@argv) = @_;
@@ -306,6 +312,7 @@ sub parse_options ($@) {
 	'f|format=s' => \$self->{_format},
 	'F|fs|cs|fieldseparator|columnseparator=s' => \$self->{_fscode},
 	'i|input=s' => sub { $self->parse_io_option('input', @_); },
+	'k|key=s' => \$self->{_key_column},
 	'log!' => \$self->{_logprog},
 	'm|median!' =>  \$self->{_median},
 	'o|output=s' => sub { $self->parse_io_option('output', @_); },
@@ -314,6 +321,7 @@ sub parse_options ($@) {
 	's|sample!' =>  \$self->{_sample},
 	'S|pre-sorted+' => \$self->{_pre_sorted},
 	'T|tmpdir|tempdir=s' => \$self->{_tmpdir},
+	'saveoutput=s' => \$self->{_save_output},
 	) or pod2usage(2);
     $self->parse_target_column(\@argv);
 }
@@ -327,7 +335,7 @@ Internal: setup, parse headers.
 =cut
 
 
-sub setup ($) {
+sub setup($) {
     my($self) = @_;
 
     pod2usage(2) if (!defined($self->{_target_column}));
@@ -338,6 +346,12 @@ sub setup ($) {
     $self->{_target_coli} = $self->{_in}->col_to_i($self->{_target_column});
     croak $self->{_prog} . ": target column " . $self->{_target_column} . " is not in input stream.\n"
 	if (!defined($self->{_target_coli}));
+    $self->{_key_coli} = undef;
+    if (defined($self->{_key_column})) {
+        $self->{_key_coli} = $self->{_in}->col_to_i($self->{_key_column});
+        croak($self->{_prog} . ": key column " . $self->{_key_column} . " is not in input stream.\n")
+	    if (!defined($self->{_key_coli}));
+    };
     my $read_fastpath_sub = $self->{_in}->fastpath_sub();
     $self->{_read_fastpath_sub} = $read_fastpath_sub;
 
@@ -349,6 +363,7 @@ sub setup ($) {
 	    push(@headers, "q$_");
 	};
     };
+    unshift(@headers, $self->{_key_column}) if (defined($self->{_key_column}));
     print STDERR "dbcolstats: pre-output setup\n" if ($self->{_debug} > 2);
     my @output_options = (-cols => \@headers);
     unshift (@output_options, -fscode => $self->{_fscode})
@@ -357,6 +372,8 @@ sub setup ($) {
     print STDERR "dbcolstats: post-output setup\n" if ($self->{_debug} > 2);
 
     if ($self->{_quantile} || $self->{_median}) {
+        croak($self->{_prog} . ": cannot currently do median or quantile with a key column\n")
+	    if (defined($self->{_key_column}));
 	$self->{_save_out_filename} = Fsdb::Support::NamedTmpfile::alloc($self->{_tmpdir});
 	# sorting needed?
 	my $save_out;
@@ -392,7 +409,7 @@ Internal: Round up to the next integer.
 
 =cut
 
-sub _round_up {
+sub _round_up($) {
     my($x) = @_;
     my($xi) = int($x);
     return ($x > $xi) ? $xi+1 : $xi;
@@ -408,7 +425,7 @@ We assume the saved output is closed before we enter.
 
 =cut
 
-sub _compute_quantile {
+sub _compute_quantile($$$) {
     my ($self, $n, $mean) = @_;
 
     return if (!($self->{_quantile} || $self->{_median}));
@@ -464,26 +481,31 @@ sub _compute_quantile {
 };
 
 
-=head2 run
+=head2 run_one_key
 
-    $filter->run();
+    $filter->run_one_key();
 
-Internal: run over each rows.
+Internal: run over each row, for a given key.
 
 =cut
-sub run ($) {
+sub run_one_key($) {
     my($self) = @_;
 
     print STDERR "dbcolstats: starting run\n" if ($self->{_debug} > 2);
 
     # xxx: should eval all this to factor out constants from runtime
     my($xf) = $self->{_target_coli};
+    my($key_column) = $self->{_key_column};
 
     my($n) = 0;
     my($sx) = 0;
     my($sxx) = 0;
     my $min;
     my $max;
+    my $key;
+    my $last_key = $self->{_holdover_key};
+    my $holdover_data = $self->{_holdover_data};
+    $self->{_holdover_key} = $self->{_holdover_data} = undef;
 
     my $fref;
     my $x;
@@ -493,9 +515,31 @@ sub run ($) {
 	my $read_fastpath_sub = $self->{_read_fastpath_sub};
 
 	my $code = q'
-	    while ($fref = &{$read_fastpath_sub}()) {
-		$x = $fref->[' . $xf . q'];
+	    while (1) {
+		if (defined($holdover_data)) {
+		    $x = $holdover_data;   # and key was set earlier
+		    $holdover_data = undef;
+		} else {
+		    $fref = &{$read_fastpath_sub}();
+		    last if (!defined($fref));
+		    $x = $fref->[' . $xf . q'];
+		';
+	if (defined($self->{_key_column})) {
+	    $code .= q'
+		    $key = $fref->[' . $self->{_key_coli} . '];
+		    if (!defined($last_key)) {
+			$last_key = $key;
+		    } elsif ($key ne $last_key) {
+			$self->{_holdover_key} = $key;
+			$self->{_holdover_data} = $x;
+			last;
+		    };
+	    ';
+	};
+	$code .= q'	       		    
+		};
 	';
+
 	$code .= 'next if ($x !~ /' . $is_numeric_regexp . "/);\n"
 	    if (!$self->{_include_non_numeric});
 	$code .= q'
@@ -619,10 +663,29 @@ sub run ($) {
 	    $out_hash{"q$_"} = $q_aref->[$_];
 	};
     };
+    if (defined($key_column)) {
+        $out_hash{$key_column} = $last_key;
+    };
 
     $self->{_out}->write_row_from_href(\%out_hash);
 }
 
+=head2 run
+
+    $filter->run();
+
+Internal: run over each row, for one or many keys.
+
+=cut
+sub run($) {
+    my($self) = @_;
+    $self->{_holdover_key} = $self->{_holdove_data} = undef;
+    for (;;) {
+	$self->run_one_key();
+	last if (!defined($self->{_holdover_key}));
+    };
+}
+
 =head1 AUTHOR and COPYRIGHT
 
 Copyright (C) 1991-2015 by John Heidemann <johnh@isi.edu>
