@@ -144,7 +144,7 @@ L<Fsdb(3)>
 
 use 5.010;
 use strict;
-use Carp qw(croak);
+use Carp qw(croak carp);
 use Pod::Usage;
 
 use Fsdb::Filter;
@@ -232,6 +232,9 @@ sub setup($) {
     if ($self->{_xargs} && $#{$self->{_inputs}} > 0) {
 	croak $self->{_prog} . ": --xargs and multiple inputs (perhaps you meant NOT --xargs?).\n";
     };
+    if (!$self->{_xargs} && $self->{_remove_inputs}) {
+	croak $self->{_prog} . ": --remove_inputs only works with --xargs.\n";
+    };
 
     if ($self->{_xargs}) {
 	croak $self->{_prog} . ": --xargs and internal error, no input.\n"
@@ -293,6 +296,10 @@ sub run($) {
 		    if ($self->{_out}->compare($this_in) ne 'identical');
 	    };
 	    $self->_run_one($this_in);
+	    if ($self->{_remove_inputs}) {
+		unlink($fref->[0]) or
+		    carp $self->{_prog} . ": --remove-inputs, but cannot remove " . $fref->[0] . "\n";
+	    };
 	};
 	croak $self->{_prog} . ": no input with --xargs\n"
 	    if ($inputs == 0);
