@@ -30,6 +30,12 @@ Does a natural, inner join on TABLE1 and TABLE2
 the specified columns.  With the C<-a> option, or 
 with C<-t outer> it will do a natural, full outer join.
 
+(Database review:
+inner joints output records only when there are matches in both tables
+and will omit records that do not match.
+Outer joins output all records from both tables, 
+filling with the empty value as needed.)
+
 By default, data will be sorted lexically,
 but the usual sorting options can be mixed with the column
 specification.
@@ -38,6 +44,8 @@ Because two tables are required,
 input is typically in files.
 Standard input is accessible by the file "-".
 
+=head1 RESOURCE REQUIREMENTS
+
 If data is already sorted, dbjoin will run more efficiently
 with the C<-S> option.
 
