@@ -355,6 +355,10 @@ sub segment_next_output($$) {
 	my $read_fh = new IO::Handle;
 	my $write_fh = new IO::Handle;
 	pipe $read_fh, $write_fh;
+	# Without these lines we suffer mojobake on unicode
+	# as shown in test base TEST/dbsort_merge_unicode.cmd.
+        binmode $read_fh, ":utf8";
+        binmode $write_fh, ":utf8";
 	$out = [ $read_fh, $write_fh ];
     } else {
 	die "internal error: dbmege.pm:segment_next_output bad output_type: $output_type\n";
@@ -542,7 +546,6 @@ sub segments_merge_one_depth($$) {
 	if ($closed && $#{$work_depth_ref} == -1 && $depth == $#{$self->{_work}}) {
 	    $output_type = 'final';
 	} elsif ($self->{_endgame} && $closed && $self->{_work_max_files}[$depth] <= $self->{_endgame_max_files}) {
-	    # endgame mode currently disabled... its' 10x slower than writing to files (!)
 	    print "# segments_merge_one_depth: endgame parallelism.\n" if ($self->{_debug});
 	    $output_type = 'ipc';
 	};
