@@ -41,6 +41,13 @@ The program will check and abort if this precondition is not met.
 With two -S's, program consumes O(1) memory, but doesn't verify
 that the data-arrival precondition is met.
 
+(Note that these semantics are exactly like
+    dbmapreduce -k KeyField -- dbcolstats ValueField
+L<dbmultistats> provides a simpler API that passes
+through statistics-specific arguments
+and is optimized when data is pre-sorted and there
+are no quarties or medians.)
+
 =head1 OPTIONS
 
 Options are the same as L<dbcolstats>.
@@ -282,19 +289,19 @@ sub setup ($) {
     # First, dbcolstats:
     #
     my @dbcolstats_argv = (qw(--nolog));
-    push (@dbcolstats_argv, '--include-non-numeric')
+    push(@dbcolstats_argv, '--include-non-numeric')
 	if (defined($self->{_include_non_numeric}));
-    push (@dbcolstats_argv, '--confidence', $self->{_confidence_fraction})
+    push(@dbcolstats_argv, '--confidence', $self->{_confidence_fraction})
 	if (defined($self->{_confidence_fraction}));
-    push (@dbcolstats_argv, '--format', $self->{_format})
+    push(@dbcolstats_argv, '--format', $self->{_format})
 	if (defined($self->{_format}));
-    push (@dbcolstats_argv, '--median')
+    push(@dbcolstats_argv, '--median')
 	if (defined($self->{_median}));
-    push (@dbcolstats_argv, '--quantile', $self->{_quantile})
+    push(@dbcolstats_argv, '--quantile', $self->{_quantile})
 	if (defined($self->{_quantile}));
-    push (@dbcolstats_argv, '--tmpdir', $self->{_tmpdir})
+    push(@dbcolstats_argv, '--tmpdir', $self->{_tmpdir})
 	if (defined($self->{_tmpdir}));
-    push (@dbcolstats_argv, '--parallelism', $self->{_max_parallelism})
+    push(@dbcolstats_argv, '--parallelism', $self->{_max_parallelism})
 	if (defined($self->{_max_parallelism}));
     # last one!
     # push (@dbcolstats_argv, $self->{_target_column});
@@ -302,6 +309,8 @@ sub setup ($) {
 
     # sigh, noclose/saveoutput didn't work
     my @dbmapreduce_argv = (qw(--nolog --noclose --copy-fs)); # --noclose --saveoutput), \$self->{_out});
+#    push(@dbmapreduce_argv, qw(--noclose --saveoutput), \$self->{_out});
+#    $self->{_child_saves_output} = 1;
     # pass input and output
     push (@dbmapreduce_argv, "--input", $self->{_input});
     push (@dbmapreduce_argv, "--output", $self->{_output});
@@ -312,6 +321,20 @@ sub setup ($) {
 	if (defined($self->{_max_parallelism}));
     push (@dbmapreduce_argv, "--key", $self->{_key_column})
 	if (defined($self->{_key_column}));
+
+    #
+    # Optimize: use dbcolstats -k if we can
+    $self->{_multi_aware_reducer} = 1;
+    $self->{_multi_aware_reducer} = undef if (defined($self->{_median}) || defined($self->{_quantile}));
+    $self->{_multi_aware_reducer} = undef if (!$self->{_pre_sorted});
+    if ($self->{_multi_aware_reducer}) {
+	push(@dbcolstats_argv, '--key', $self->{_key_column});
+	push(@dbmapreduce_argv, '--multiple-ok');
+	$self->{_child_saves_output} = 1;
+	push(@dbmapreduce_argv, '--noclose', '--saveoutput', \$self->{_out});
+    };
+
+
     my $dbcolstats_code = 'dbcolstats(';
     foreach (@dbcolstats_argv) {
 	$dbcolstats_code .= "'$_', ";
@@ -319,6 +342,8 @@ sub setup ($) {
     $dbcolstats_code .= "'" . $self->{_target_column} . "')";
     push (@dbmapreduce_argv, '--code', $dbcolstats_code);
 
+    print join(" ", @dbmapreduce_argv) if ($self->{_debug});    
+
     $self->{_mapreducer} = new Fsdb::Filter::dbmapreduce(@dbmapreduce_argv);
     $self->{_mapreducer}->setup;
 }
@@ -351,19 +376,23 @@ sub finish ($) {
     # xxx: hack hack hack
     # --saveoutput didn't work, so fake it up here
     my $post = "#" . $self->compute_program_log() . "\n";
-    if (ref($self->{_output}) =~ /^Fsdb::IO/) {	
-	$self->{_output}->write_comment($post);
-	$self->{_output}->close if ($self->{_close});
-    } elsif (ref($self->{_output}) =~ /^Fsdb::BoundedQueue/) {	
-	$self->{_output}->enqueue($post);
-	$self->{_output}->enqueue(undef)  if ($self->{_close});
-    } elsif ($self->{_output} eq '-') {
-	# stdout
-	print $post;
+    if ($self->{_child_saves_output}) {
+        $self->SUPER::finish();
     } else {
-	# assume file handle
-	$self->{_output}->print($post);
-	$self->{_output}->close if ($self->{_close});
+        if (ref($self->{_output}) =~ /^Fsdb::IO/) {	
+    	    $self->{_output}->write_comment($post);
+	    $self->{_output}->close if ($self->{_close});
+        } elsif (ref($self->{_output}) =~ /^Fsdb::BoundedQueue/) {	
+	    $self->{_output}->enqueue($post);
+	    $self->{_output}->enqueue(undef)  if ($self->{_close});
+        } elsif ($self->{_output} eq '-') {
+	    # stdout
+	    print $post;
+        } else {
+	    # assume file handle
+	    $self->{_output}->print($post);
+	    $self->{_output}->close if ($self->{_close});
+        };
     };
 }
 
